/*
 * Copyright (C) 2014 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.android.systemui.qs;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.drawable.Drawable;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.os.UserHandle;
import android.provider.Settings;
import android.text.SpannableStringBuilder;
import android.text.method.LinkMovementMethod;
import android.text.style.ClickableSpan;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.widget.ImageView;
import android.widget.TextView;

import com.android.systemui.FontSizeUtils;
import com.android.systemui.R;
import com.android.systemui.statusbar.phone.QSTileHost;
import com.android.systemui.statusbar.phone.SystemUIDialog;
import com.android.systemui.statusbar.policy.SecurityController;

import static android.provider.Settings.ACTION_VPN_SETTINGS;

public class QSFooter implements OnClickListener, DialogInterface.OnClickListener {
    protected static final String TAG = "QSFooter";
    protected static final boolean DEBUG = Log.isLoggable(TAG, Log.DEBUG);

    private final View mRootView;
    private final TextView mFooterText;
    private final ImageView mFooterIcon;
    private final Context mContext;
    private final Callback mCallback = new Callback();

    private SecurityController mSecurityController;
    private AlertDialog mDialog;
    private QSTileHost mHost;
    protected Handler mHandler;
    private final Handler mMainHandler;

    private boolean mIsVisible;
    private boolean mIsIconVisible;
    private CharSequence mFooterTextContent = null;
    private int mFooterIconId;

    public QSFooter(QSPanel qsPanel, Context context) {
        mRootView = LayoutInflater.from(context)
                .inflate(R.layout.quick_settings_footer, qsPanel, false);
        mRootView.setOnClickListener(this);
        mFooterText = (TextView) mRootView.findViewById(R.id.footer_text);
        mFooterIcon = (ImageView) mRootView.findViewById(R.id.footer_icon);
        mFooterIconId = R.drawable.ic_qs_vpn;
        mContext = context;
        mMainHandler = new Handler(Looper.getMainLooper());
    }

    public void setHostEnvironment(QSTileHost host, SecurityController securityController,
            Looper looper) {
        mHost = host;
        mSecurityController = securityController;
        mHandler = new H(looper);
    }

    public void setListening(boolean listening) {
        if (listening) {
            mSecurityController.addCallback(mCallback);
        } else {
            mSecurityController.removeCallback(mCallback);
        }
    }

    public void onConfigurationChanged() {
        FontSizeUtils.updateFontSize(mFooterText, R.dimen.qs_tile_text_size);
    }

    public View getView() {
        return mRootView;
    }

    public boolean hasFooter() {
        return mRootView.getVisibility() != View.GONE;
    }

    @Override
    public void onClick(View v) {
        mHandler.sendEmptyMessage(H.CLICK);
    }

    private void handleClick() {
        mHost.collapsePanels();
        // TODO: Delay dialog creation until after panels are collapsed.
        createDialog();
    }

    public void refreshState() {
        mHandler.sendEmptyMessage(H.REFRESH_STATE);
    }

    private void handleRefreshState() {
        mIsIconVisible = mSecurityController.isVpnEnabled();
        if (mSecurityController.isDeviceManaged()) {
            final CharSequence organizationName =
                    mSecurityController.getDeviceOwnerOrganizationName();
            if (organizationName != null) {
                mFooterTextContent = mContext.getResources().getString(
                        R.string.do_disclosure_with_name, organizationName);
            } else {
                mFooterTextContent =
                        mContext.getResources().getString(R.string.do_disclosure_generic);
            }
            mIsVisible = true;
        } else {
            boolean isBranded = mSecurityController.isVpnBranded();
            mFooterTextContent = mContext.getResources().getText(
                    isBranded ? R.string.branded_vpn_footer : R.string.vpn_footer);
            // Update the VPN footer icon, if needed.
            int footerIconId = isBranded ? R.drawable.ic_qs_branded_vpn : R.drawable.ic_qs_vpn;
            if (mFooterIconId != footerIconId) {
                mFooterIconId = footerIconId;
                mMainHandler.post(mUpdateIcon);
            }
            mIsVisible = mIsIconVisible;
        }
        mMainHandler.post(mUpdateDisplayState);
    }

    @Override
    public void onClick(DialogInterface dialog, int which) {
        if (which == DialogInterface.BUTTON_NEGATIVE) {
            final Intent settingsIntent = new Intent(ACTION_VPN_SETTINGS);
            mHost.startActivityDismissingKeyguard(settingsIntent);
        }
    }

    private void createDialog() {
        final String deviceOwnerPackage = mSecurityController.getDeviceOwnerName();
        final String profileOwnerPackage = mSecurityController.getProfileOwnerName();
        final String primaryVpn = mSecurityController.getPrimaryVpnName();
        final String profileVpn = mSecurityController.getProfileVpnName();
        final CharSequence deviceOwnerOrganization =
                mSecurityController.getDeviceOwnerOrganizationName();
        boolean hasProfileOwner = mSecurityController.hasProfileOwner();
        boolean isBranded = deviceOwnerPackage == null && mSecurityController.isVpnBranded();

        mDialog = new SystemUIDialog(mContext);
        if (!isBranded) {
            mDialog.setTitle(getTitle(deviceOwnerPackage));
        }
        mDialog.setMessage(getMessage(deviceOwnerPackage, profileOwnerPackage, primaryVpn,
                profileVpn, deviceOwnerOrganization, hasProfileOwner, isBranded));

        mDialog.setButton(DialogInterface.BUTTON_POSITIVE, getPositiveButton(isBranded), this);
        if (mSecurityController.isVpnEnabled() && !mSecurityController.isVpnRestricted()) {
            mDialog.setButton(DialogInterface.BUTTON_NEGATIVE, getSettingsButton(), this);
        }

        // Make the link "learn more" clickable.
        // TODO: Reaching into SystemUIDialog's internal View hierarchy is ugly and error-prone.
        // This is a temporary solution. b/33126622 will introduce a custom View hierarchy for this
        // dialog, allowing us to set the movement method on the appropriate View without any
        // knowledge of SystemUIDialog's internals.
        mDialog.create();
        ((TextView) mDialog.getWindow().findViewById(com.android.internal.R.id.message))
                .setMovementMethod(new LinkMovementMethod());

        mDialog.show();
    }

    private String getSettingsButton() {
        return mContext.getString(R.string.status_bar_settings_settings_button);
    }

    private String getPositiveButton(boolean isBranded) {
        return mContext.getString(isBranded ? android.R.string.ok : R.string.quick_settings_done);
    }

    protected CharSequence getMessage(String deviceOwnerPackage, String profileOwnerPackage,
            String primaryVpn, String profileVpn, CharSequence deviceOwnerOrganization,
            boolean hasProfileOwner, boolean isBranded) {
        if (deviceOwnerPackage != null) {
            final SpannableStringBuilder message = new SpannableStringBuilder();
            if (deviceOwnerOrganization != null) {
                message.append(mContext.getString(
                        R.string.monitoring_description_do_header_with_name,
                        deviceOwnerOrganization, deviceOwnerPackage));
            } else {
                message.append(mContext.getString(R.string.monitoring_description_do_header_generic,
                        deviceOwnerPackage));
            }
            message.append("\n\n");
            message.append(mContext.getString(R.string.monitoring_description_do_body));
            if (primaryVpn != null) {
                message.append("\n\n");
                message.append(mContext.getString(R.string.monitoring_description_do_body_vpn,
                        primaryVpn));
            }
            message.append(mContext.getString(
                    R.string.monitoring_description_do_learn_more_separator));
            message.append(mContext.getString(R.string.monitoring_description_do_learn_more),
                    new EnterprisePrivacySpan(), 0);
            return message;
        } else if (primaryVpn != null) {
            if (profileVpn != null) {
                return mContext.getString(R.string.monitoring_description_app_personal_work,
                        profileOwnerPackage, profileVpn, primaryVpn);
            } else {
                if (isBranded) {
                    return mContext.getString(R.string.branded_monitoring_description_app_personal,
                            primaryVpn);
                } else {
                    return mContext.getString(R.string.monitoring_description_app_personal,
                            primaryVpn);
                }
            }
        } else if (profileVpn != null) {
            return mContext.getString(R.string.monitoring_description_app_work,
                    profileOwnerPackage, profileVpn);
        } else if (profileOwnerPackage != null && hasProfileOwner) {
            return mContext.getString(R.string.monitoring_description_device_owned,
                    profileOwnerPackage);
        } else {
            // No device owner, no personal VPN, no work VPN, no user owner. Why are we here?
            return null;
        }
    }

    private int getTitle(String deviceOwner) {
        if (deviceOwner != null) {
            return R.string.monitoring_title_device_owned;
        } else {
            return R.string.monitoring_title;
        }
    }

    private final Runnable mUpdateIcon = new Runnable() {
        @Override
        public void run() {
            mFooterIcon.setImageResource(mFooterIconId);
        }
    };

    private final Runnable mUpdateDisplayState = new Runnable() {
        @Override
        public void run() {
            if (mFooterTextContent != null) {
                mFooterText.setText(mFooterTextContent);
            }
            mRootView.setVisibility(mIsVisible ? View.VISIBLE : View.GONE);
            mFooterIcon.setVisibility(mIsIconVisible ? View.VISIBLE : View.INVISIBLE);
        }
    };

    private class Callback implements SecurityController.SecurityControllerCallback {
        @Override
        public void onStateChanged() {
            refreshState();
        }
    }

    private class H extends Handler {
        private static final int CLICK = 0;
        private static final int REFRESH_STATE = 1;

        private H(Looper looper) {
            super(looper);
        }

        @Override
        public void handleMessage(Message msg) {
            String name = null;
            try {
                if (msg.what == REFRESH_STATE) {
                    name = "handleRefreshState";
                    handleRefreshState();
                } else if (msg.what == CLICK) {
                    name = "handleClick";
                    handleClick();
                }
            } catch (Throwable t) {
                final String error = "Error in " + name;
                Log.w(TAG, error, t);
                mHost.warn(error, t);
            }
        }
    }

    protected class EnterprisePrivacySpan extends ClickableSpan {
        @Override
        public void onClick(View widget) {
            final Intent intent = new Intent(Settings.ACTION_ENTERPRISE_PRIVACY_SETTINGS);
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP);
            mDialog.dismiss();
            mContext.startActivity(intent);
        }

        @Override
        public boolean equals(Object object) {
            return object instanceof EnterprisePrivacySpan;
        }
    }
}
